"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Indicator = void 0;
var React = _interopRequireWildcard(require("react"));
var _reactNative = require("react-native");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
function _extends() { return _extends = Object.assign ? Object.assign.bind() : function (n) { for (var e = 1; e < arguments.length; e++) { var t = arguments[e]; for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]); } return n; }, _extends.apply(null, arguments); } /* eslint-disable react-native/no-inline-styles */ /**
 * @format
 */
const Indicator = props => {
  const {
    d,
    sc,
    horizontal,
    indSize,
    diff,
    inverted,
    scrollRefs,
    contentSize,
    visibleSize,
    parentPos,
    locStyle,
    indStyle
  } = props;

  // The indicator offset (the distance between the top or left of the
  // indicator to the top or left of the content boundary) the moment when it
  // starts to be dragged. We need this offset to properly compute the
  // indicator offset when it is being dragged.
  const indOffsetOnMove = React.useRef(0);

  // Use ref to make values of the props or any value that might be changed
  // upon refresh available inside animated value callbacks. The values in
  // animated callbacks do not update upon any state update.
  const refs = {
    horizontal: React.useRef(false),
    inverted: React.useRef(false),
    actualIndSize: React.useRef(0),
    diff: React.useRef(0),
    contentSize: React.useRef(0),
    visibleSize: React.useRef(0)
  };
  refs.horizontal.current = horizontal;
  refs.inverted.current = inverted;
  refs.diff.current = diff;
  refs.contentSize.current = contentSize;
  refs.visibleSize.current = visibleSize;
  const pan = React.useRef(new _reactNative.Animated.ValueXY()).current;
  pan.addListener(({
    x,
    y
  }) => {
    // x and y is the delta x and delta y when the indicator is being dragged.
    // Thus the total amount of indicator offset is the initial offset plus the
    // delta value.
    // Note that we clamp on how much indicator can be dragged. It cannot
    // be dragged beyond the scrollable component
    // Also note that when Flatlist is inverted, so is the delta value, because
    // we want to keep indicatorOffset always positive. It always increases
    // as the top or left (bottom or right) of the indicator moves away from
    // the top or left (bottom or right) of the parent bound.
    const indicatorOffset = Math.min(Math.max((refs.horizontal.current ? refs.inverted.current ? -x : x : refs.inverted.current ? -y : y) + indOffsetOnMove.current, 0), refs.diff.current);
    d.setValue(indicatorOffset);

    // Compute the content offset to manually scroll the scrollable component
    // Note that animated is set to false because the pan itself is an
    // animated value. As y is being changed, there is no need to add further
    // animation. The animation is needed when we supply the end offset to
    // the scrollable component. In the current case, we are not assigning
    // the end offset. Rather, we are assigning the current offset.
    const contentOffset = indicatorOffset * refs.contentSize.current / refs.visibleSize.current;
    if (scrollRefs.FlatList.current) {
      scrollRefs.FlatList.current.scrollToOffset({
        offset: contentOffset,
        animated: false
      });
    } else if (scrollRefs.ScrollView.current) {
      scrollRefs.ScrollView.current.scrollTo(refs.horizontal.current ? {
        x: contentOffset,
        animated: false
      } : {
        y: contentOffset,
        animated: false
      });
    }
  });
  const panResponder = React.useRef(_reactNative.PanResponder.create({
    onMoveShouldSetPanResponder: () => true,
    onPanResponderGrant: evt => {
      // top or left of indicator to the top or left of parent boundary
      const indStartToParentStart = refs.horizontal.current ? evt.nativeEvent.pageX - evt.nativeEvent.locationX - parentPos.pageX : evt.nativeEvent.pageY - evt.nativeEvent.locationY - parentPos.pageY;
      // If inverted, we need to compute the bottom or right of indicator
      // to the bottom or right of parent boundary
      indOffsetOnMove.current = refs.inverted.current ? refs.visibleSize.current - indStartToParentStart - refs.actualIndSize.current : indStartToParentStart;
    },
    onPanResponderMove: _reactNative.Animated.event([null, {
      dy: pan.y,
      dx: pan.x
    }], {
      useNativeDriver: false
    })
  })).current;

  // Use the indicator offset to interpolate the actual travel on x or y
  // coordinate
  // Note that the max travel distance is diff
  const move = d.interpolate({
    inputRange: [0, diff],
    outputRange: inverted ? [diff, 0] : [0, diff],
    extrapolate: 'extend'
  });
  // interpolate the scale need to shrink by the indicator to scaleX or scaleY.
  // Note that the range of shrink scale is from 0 (completely shrunk,
  // invisible) to 1 (no shrink)
  const shrink = sc.interpolate({
    inputRange: [0, 1],
    outputRange: [0, 1],
    extrapolate: 'clamp'
  });
  return /*#__PURE__*/React.createElement(_reactNative.Animated.View, _extends({
    style: {
      ...indStyle,
      ...locStyle,
      position: 'absolute',
      height: horizontal ? indStyle.width : indSize,
      width: horizontal ? indSize : indStyle.width,
      transform: horizontal ? [{
        translateX: move
      }, {
        scaleX: shrink
      }] : [{
        translateY: move
      }, {
        scaleY: shrink
      }]
    },
    onLayout: e => {
      // The actual size of the indicator is different from the given indSize
      // due to size changes in the parent components
      refs.actualIndSize.current = horizontal ? e.nativeEvent.layout.width : e.nativeEvent.layout.height;
    }
  }, panResponder.panHandlers));
};
exports.Indicator = Indicator;
//# sourceMappingURL=Indicator.js.map