"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ScrollIndicator = void 0;
var React = _interopRequireWildcard(require("react"));
var _reactNative = require("react-native");
var _Indicator = require("./Indicator");
var _functions = require("./functions");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
function _extends() { return _extends = Object.assign ? Object.assign.bind() : function (n) { for (var e = 1; e < arguments.length; e++) { var t = arguments[e]; for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]); } return n; }, _extends.apply(null, arguments); } /**
 * @format
 */
const ScrollIndicator = props => {
  const {
    target,
    targetProps,
    position,
    horizontal,
    persistentScrollbar,
    indStyle,
    containerStyle
  } = props;

  // total size of the content if rendered
  const [contentSize, setContentSize] = React.useState(1);
  // size of the visible part of the content, i.e., size of the scroll view
  // itself.
  const [visibleSize, setVisibleSize] = React.useState(0);
  // the size orthogonal to visibleSize, for use in computing the position of
  // the indicator
  const [orthSize, setOrthSize] = React.useState(0);

  // parent view position. Parent view is the component that holds both the
  // scroll indicator and the scrollable component
  const parentRef = React.useRef(null);
  const [parentPos, setParentPos] = React.useState({
    pageX: 0,
    pageY: 0,
    ready: false
  });

  // scroll container refs. Use this to manually scroll the scrollable
  // component when dragging the indicator
  const scrollRefs = {
    FlatList: React.useRef(null),
    ScrollView: React.useRef(null)
  };

  // height or width of the indicator, if it is vertical or horizontal,
  // respectively. If there is more content than visible on the view, the
  // proportion of indSize to visibleSize is the same as the visibleSize to
  // contentSize. Otherwise, set the indSize the same as visibleSize.
  const indSize = contentSize > visibleSize ? visibleSize * visibleSize / contentSize : visibleSize;

  // the amount of distance the indicator needs to travel during scrolling
  // without any shrinking
  const diff = visibleSize > indSize ? visibleSize - indSize : 1;

  // distance that the top or left of the indicator needs to travel in
  // accordance with scrolling
  const d = React.useRef(new _reactNative.Animated.Value(0)).current;
  // the scale that the indicator needs to shrink if scrolling beyond the end
  const sc = React.useRef(new _reactNative.Animated.Value(1)).current;

  /****************************************************
   * Callbacks shared by both Flatlist and Scrollview
   ****************************************************/
  const configContentSize = (w, h) => {
    // total size of the content
    setContentSize(horizontal ? w : h);
  };
  const configVisibleSize = e => {
    // layout of the visible part of the scroll view
    setVisibleSize(horizontal ? e.nativeEvent.layout.width : e.nativeEvent.layout.height);
    setOrthSize(horizontal ? e.nativeEvent.layout.height : e.nativeEvent.layout.width);
  };
  const configOnScroll = e => {
    /**
     * obtain contentOffset, which is the distance from the top or left
     * of the content to the top or left of the scroll view.
     * contentOffset gets bigger if user scrolls up or left (i.e.
     * content goes up or left),
     * otherwise smaller. It is possible for contentOffset to be
     * negative, if user scrolls down or right until there is empty
     * space above or to the left of the content.
     * indicatorOffset is computed similarly to indSize, in which the
     * proportion of the amount of distance to travel by the indicator
     * to the container size is the same as the proportion of the
     * container size to total size.
     */
    const indicatorOffset = (horizontal ? e.nativeEvent.contentOffset.x : e.nativeEvent.contentOffset.y) * visibleSize / contentSize;
    d.setValue(indicatorOffset);
    /**
     * What we desire is that when the indicator touches the edge, it
     * shrinks in size while maintaining the contact to the edge if
     * user scrolls in the same direction further.
     * If we don't move the indicator, after shrinking, there will be a
     * gap of size
     *
     * (indSize - indSize * sc) / 2
     *
     * between the end of the indicator and the edge of the container.
     * To make the end of the indicator maintain contact to the edge,
     * the indicator must move in the same rate as the gap appears.
     *
     * If we scroll down or right, we have the following relationship
     *
     * indicatorOffset = diff + (indSize - indSize * sc) / 2
     *
     * If we scroll up or left, we have a slightly different
     * relationship
     *
     * indicatorOffset = (indSize - indSize * sc) / 2
     *
     * From these two relationship, we can compute sc based on
     * indicatorOffset.
     */
    sc.setValue(indicatorOffset >= 0 ? (indSize + 2 * diff - 2 * indicatorOffset) / indSize : (indSize + 2 * indicatorOffset) / indSize);
  };
  return /*#__PURE__*/React.createElement(_reactNative.View, {
    ref: parentRef,
    style: containerStyle,
    onLayout: () => {
      if (parentRef.current) {
        var _parentRef$current;
        (_parentRef$current = parentRef.current) === null || _parentRef$current === void 0 || _parentRef$current.measure((_1, _2, _3, _4, pageX, pageY) => {
          setParentPos({
            pageX: pageX,
            pageY: pageY,
            ready: true
          });
        });
      }
    }
  }, target === 'FlatList' ? /*#__PURE__*/React.createElement(_reactNative.FlatList, _extends({}, targetProps, {
    ref: scrollRefs.FlatList,
    horizontal: horizontal,
    showsVerticalScrollIndicator: false,
    showsHorizontalScrollIndicator: false,
    onContentSizeChange: configContentSize,
    onLayout: e => {
      configVisibleSize(e);
      if ('onLayout' in targetProps && typeof targetProps.onLayout === 'function') {
        targetProps.onLayout(e);
      }
    },
    scrollEventThrottle: 16,
    onScroll: e => {
      configOnScroll(e);
      if ('onScroll' in targetProps && typeof targetProps.onScroll === 'function') {
        targetProps.onScroll(e);
      }
    }
  })) :
  /*#__PURE__*/
  // The logic for ScrollView is exactly the same as FlatList
  React.createElement(_reactNative.ScrollView, _extends({}, targetProps, {
    ref: scrollRefs.ScrollView,
    horizontal: horizontal,
    showsVerticalScrollIndicator: false,
    showsHorizontalScrollIndicator: false,
    onContentSizeChange: configContentSize,
    onLayout: e => {
      configVisibleSize(e);
      if ('onLayout' in targetProps && typeof targetProps.onLayout === 'function') {
        targetProps.onLayout(e);
      }
    },
    scrollEventThrottle: 16,
    onScroll: e => {
      configOnScroll(e);
      if ('onScroll' in targetProps && typeof targetProps.onScroll === 'function') {
        targetProps.onScroll(e);
      }
    }
  }), props.children), (persistentScrollbar || indSize < visibleSize) && parentPos.ready && /*#__PURE__*/React.createElement(_Indicator.Indicator, {
    d: d,
    sc: sc,
    horizontal: horizontal,
    indSize: indSize,
    diff: diff,
    inverted: 'inverted' in targetProps ? typeof targetProps.inverted === 'boolean' ? targetProps.inverted : false : false,
    scrollRefs: scrollRefs,
    contentSize: contentSize,
    visibleSize: visibleSize,
    parentPos: parentPos,
    locStyle: (0, _functions.getLocStyle)(horizontal, position, orthSize, indStyle.width),
    indStyle: indStyle
  }));
};
exports.ScrollIndicator = ScrollIndicator;
//# sourceMappingURL=ScrollIndicator.js.map